Package: RJMCMCNucleosomes
Version: 1.37.0
Date: 2025-07-28
Title: Bayesian hierarchical model for genome-wide nucleosome
        positioning with high-throughput short-read data (MNase-Seq)
Description: This package does nucleosome positioning using informative
    Multinomial-Dirichlet prior in a t-mixture with reversible jump
    estimation of nucleosome positions for genome-wide profiling.
Authors@R: c(person("Pascal", "Belleau",
    email="pascal_belleau@hotmail.com", role=c("aut")),
    person("Rawane", "Samb",
    email="rawane.samb.1@ulaval.ca", role=c("aut")),
    person("Astrid", "Deschênes",
    email="adeschen@hotmail.com",
    role=c("cre","aut")),
    person("Khader", "Khadraoui",
    email="Khader.Khadraoui@mat.ulaval.ca", role=c("aut")),
    person("Lajmi", "Lakhal-Chaieb",
    email="Lajmi.Lakhal@mat.ulaval.ca;", role=c("aut")),
    person("Arnaud", "Droit",
    email="arnaud.droit@crchuq.ulaval.ca", role=c("aut")))
Depends: R (>= 3.5), IRanges, GenomicRanges
Imports: Rcpp (>= 0.12.5), consensusSeekeR, BiocGenerics, Seqinfo,
        S4Vectors (>= 0.23.10), BiocParallel, stats, graphics, methods,
        grDevices
Suggests: BiocStyle, knitr, rmarkdown, nucleoSim, RUnit
LinkingTo: Rcpp
SystemRequirements: Rcpp
Encoding: UTF-8
License: Artistic-2.0
URL: https://github.com/ArnaudDroitLab/RJMCMCNucleosomes
BugReports: https://github.com/ArnaudDroitLab/RJMCMCNucleosomes/issues
VignetteBuilder: knitr
biocViews: BiologicalQuestion, ChIPSeq, NucleosomePositioning,
        Software, StatisticalMethod, Bayesian, Sequencing, Coverage
RoxygenNote: 6.0.1
git_url: https://git.bioconductor.org/packages/RJMCMCNucleosomes
git_branch: devel
git_last_commit: 510ac6e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: yes
Packaged: 2026-05-18 23:14:12 UTC; biocbuild
Author: Pascal Belleau [aut],
  Rawane Samb [aut],
  Astrid Deschênes [cre, aut],
  Khader Khadraoui [aut],
  Lajmi Lakhal-Chaieb [aut],
  Arnaud Droit [aut]
Maintainer: Astrid Deschênes <adeschen@hotmail.com>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-19 05:06:09 UTC; unix
