# Generated by roxygen2: do not edit by hand

S3method(print,rjmcmcNucleosomes)
S3method(print,rjmcmcNucleosomesBeforeAndAfterPostTreatment)
S3method(print,rjmcmcNucleosomesMerge)
export(mergeAllRDSFilesFromDirectory)
export(mergeRDSFiles)
export(plotNucleosomes)
export(postTreatment)
export(rjmcmc)
export(rjmcmcCHR)
export(segmentation)
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,sapply)
importFrom(BiocGenerics,start)
importFrom(BiocParallel,SnowParam)
importFrom(BiocParallel,bplapply)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,findOverlaps)
importFrom(GenomicRanges,strand)
importFrom(IRanges,IRanges)
importFrom(IRanges,coverage)
importFrom(Rcpp,evalCpp)
importFrom(S4Vectors,isSingleInteger)
importFrom(S4Vectors,isSingleNumber)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,runValue)
importFrom(S4Vectors,splitAsList)
importFrom(S4Vectors,subjectHits)
importFrom(Seqinfo,Seqinfo)
importFrom(Seqinfo,genome)
importFrom(Seqinfo,seqinfo)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqnames)
importFrom(consensusSeekeR,findConsensusPeakRegions)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(methods,is)
importFrom(stats,aggregate)
importFrom(stats,setNames)
useDynLib(RJMCMCNucleosomes)
